import { ContentRedirectionVisitor } from '../proxy/ContentRedirectionVisitor';
import { DialogProxyTools } from '../proxy/DialogProxyTools';
import { DialogRedirectionVisitor } from '../proxy/DialogRedirectionVisitor';
import { DialogVisitor } from '../proxy/DialogVisitor';
import { LargePropertyVisitor } from '../proxy/LargePropertyVisitor';
import { RecordSetVisitor } from '../proxy/RecordSetVisitor';
import { RecordVisitor } from '../proxy/RecordVisitor';
import { SessionVisitor } from '../proxy/SessionVisitor';
import { storage } from '../storage';
import { Log } from '../util/Log';
import { BriefcaseVisitor } from './BriefcaseVisitor';
import { SdaGetBriefcaseRecordJsonSample } from './samples/SdaGetBriefcaseRecordJsonSample';
import { SdaDialogDelegateStateVisitor } from './SdaDialogDelegateStateVisitor';
export class SdaDialogDelegateTools {
    static constructAddToBriefcaseNullRedirection(tenantId, sessionId, referringDialogId) {
        const nullRedirectionId = DialogProxyTools.constructNullRedirectionId();
        return {
            tenantId: tenantId,
            referringObject: {
                dialogMode: 'LIST',
                dialogAlias: 'Workpackage_AddToBriefcase_FORM',
                actionId: 'alias_AddToBriefcase',
                type: 'hxgn.api.dialog.ReferringDialog',
                dialogId: referringDialogId
            },
            refreshNeeded: true,
            sessionId: sessionId,
            id: nullRedirectionId,
            type: 'hxgn.api.dialog.NullRedirection'
        };
    }
    static constructEnterOfflineModeNullRedirection(tenantId, sessionId) {
        return this.constructOfflineModeNullRedirection(tenantId, sessionId, this.ENTER_OFFLINE_MODE_MENU_ACTION_ID);
    }
    static constructExitOfflineModeNullRedirection(tenantId, sessionId) {
        return this.constructOfflineModeNullRedirection(tenantId, sessionId, this.EXIT_OFFLINE_MODE_MENU_ACTION_ID);
    }
    static constructOfflineLogoutResponse(sessionId) {
        return {
            sessionId: sessionId,
            type: this.SESSION_ID_MODEL_TYPE
        };
    }
    static constructRemoveFromBriefcaseNullRedirection(tenantId, sessionId, referringDialogId) {
        const nullRedirectionId = DialogProxyTools.constructNullRedirectionId();
        return {
            tenantId: tenantId,
            referringObject: {
                dialogMode: 'LIST',
                dialogAlias: 'Workpackage_RemoveFromBriefcase_FORM',
                actionId: 'alias_RemoveFromBriefcase',
                type: 'hxgn.api.dialog.ReferringDialog',
                dialogId: referringDialogId
            },
            refreshNeeded: true,
            sessionId: sessionId,
            id: nullRedirectionId,
            type: 'hxgn.api.dialog.NullRedirection'
        };
    }
    static getOfflineDocumentsDialogListId(workPackageId) {
        return this.OFFLINE_DOCUMENTS_DIALOG_LIST_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineDocumentsDialogPropertiesId(workPackageId) {
        return this.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineDocumentsDialogRootId(workPackageId) {
        return this.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineTagsDialogListId(workPackageId) {
        return this.OFFLINE_TAGS_DIALOG_LIST_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineTagsDialogPropertiesId(workPackageId) {
        return this.OFFLINE_TAGS_DIALOG_PROPERTIES_ID.replace('${workPackageId}', workPackageId);
    }
    static getOfflineTagsDialogRootId(workPackageId) {
        return this.OFFLINE_TAGS_DIALOG_ROOT_ID.replace('${workPackageId}', workPackageId);
    }
    static isBriefcaseWorkbenchActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostWorkbenchAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostMenuActionPath(resourcePathElems);
        return pathFields.actionId === SdaDialogDelegateTools.BRIEFCASE_WORKBENCH_ACTION_ID;
    }
    static isEnterOfflineModeMenuActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostMenuAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostMenuActionPath(resourcePathElems);
        return pathFields.actionId === SdaDialogDelegateTools.ENTER_OFFLINE_MODE_MENU_ACTION_ID;
    }
    static isExitOfflineModeMenuActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostMenuAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostMenuActionPath(resourcePathElems);
        return pathFields.actionId === SdaDialogDelegateTools.EXIT_OFFLINE_MODE_MENU_ACTION_ID;
    }
    static isOfflineBriefcaseCommentsRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostRecords(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostRecordsPath(resourcePathElems);
        return SdaDialogDelegateTools.isOfflineBriefcaseCommentsDialogId(pathFields.dialogId);
    }
    static isOfflineBriefcaseDialogId(dialogId) {
        return dialogId === this.OFFLINE_BRIEFCASE_DIALOG_ROOT_ID;
    }
    static isOfflineBriefcaseDetailsDialogId(dialogId) {
        return dialogId === this.OFFLINE_BRIEFCASE_DIALOG_DETAILS_ID;
    }
    static isOfflineBriefcaseWorkPackagesDialogId(dialogId) {
        return dialogId === this.OFFLINE_BRIEFCASE_DIALOG_WORK_PACKAGES_ID;
    }
    static isOfflineBriefcaseWorkPackagesRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostRecords(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostRecordsPath(resourcePathElems);
        return SdaDialogDelegateTools.isOfflineBriefcaseWorkPackagesDialogId(pathFields.dialogId);
    }
    static isOfflineBriefcaseCommentsDialogId(dialogId) {
        return dialogId === this.OFFLINE_BRIEFCASE_DIALOG_COMMENTS_ID;
    }
    static isOfflineDocumentContentRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostSessionContent(resourcePathElems)) {
            return false;
        }
        return true;
    }
    static isOfflineDocumentOpenLatestFileMenuActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostMenuAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostMenuActionPath(resourcePathElems);
        return pathFields.actionId === this.OPEN_LATEST_FILE_MENU_ACTION_ID;
    }
    static isOfflineDocumentsListRecordSetRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostRecords(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostRecordsPath(resourcePathElems);
        return pathFields.dialogId.startsWith(this.OFFLINE_DOCUMENTS_DIALOG_LIST_ID_PREFIX);
    }
    static isOfflineDocumentsPropertiesDialogId(dialogId) {
        return dialogId.startsWith(this.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID_PREFIX);
    }
    static isOfflineDocumentsRootDialogRequest(resourcePathElems) {
        if (!DialogProxyTools.isGetDialog(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructGetDialogPath(resourcePathElems);
        return pathFields.dialogId.startsWith(this.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID_PREFIX);
    }
    static isOfflineShowTagsMenuActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostMenuAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostMenuActionPath(resourcePathElems);
        return pathFields.actionId === this.SHOW_TAGS_MENU_ACTION_ID;
    }
    static isOfflineTagsListRecordSetRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostRecords(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostRecordsPath(resourcePathElems);
        return pathFields.dialogId.startsWith(this.OFFLINE_TAGS_DIALOG_LIST_ID_PREFIX);
    }
    static isOfflineTagsPropertiesDialogId(dialogId) {
        return dialogId.startsWith(this.OFFLINE_TAGS_DIALOG_PROPERTIES_ID_PREFIX);
    }
    static isOfflineTagsRootDialogRequest(resourcePathElems) {
        if (!DialogProxyTools.isGetDialog(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructGetDialogPath(resourcePathElems);
        return pathFields.dialogId.startsWith(this.OFFLINE_TAGS_DIALOG_ROOT_ID_PREFIX);
    }
    static isOfflineWorkPackagesListRecordSetRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostRecords(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostRecordsPath(resourcePathElems);
        return pathFields.dialogId === this.OFFLINE_WORK_PACKAGES_DIALOG_LIST_ID;
    }
    static isOfflineWorkPackagesOpenMenuActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostMenuAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostMenuActionPath(resourcePathElems);
        return (pathFields.dialogId === this.OFFLINE_WORK_PACKAGES_DIALOG_LIST_ID &&
            pathFields.actionId === this.OPEN_MENU_ACTION_ID);
    }
    static isOfflineWorkPackagesRootDialogRequest(resourcePathElems) {
        if (!DialogProxyTools.isGetDialog(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructGetDialogPath(resourcePathElems);
        return pathFields.dialogId === this.OFFLINE_WORK_PACKAGES_DIALOG_ROOT_ID;
    }
    static isWorkPackagesAddToBriefcaseMenuActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostMenuAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostMenuActionPath(resourcePathElems);
        return pathFields.actionId === SdaDialogDelegateTools.ADD_TO_BRIEFCASE_MENU_ACTION_ID;
    }
    static isWorkPackagesListRecordSet(resourcePathElems, jsonObject) {
        if (!DialogProxyTools.isPostRecords(resourcePathElems)) {
            return false;
        }
        if (!jsonObject || !jsonObject.type || !jsonObject.dialogName) {
            return false;
        }
        return (jsonObject.type === this.RECORD_SET_MODEL_TYPE &&
            jsonObject.dialogName === this.WORK_PACKAGES_LIST_DIALOG_NAME);
    }
    static isWorkPackagesRemoveFromBriefcaseMenuActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostMenuAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostMenuActionPath(resourcePathElems);
        return pathFields.actionId === SdaDialogDelegateTools.REMOVE_FROM_BRIEFCASE_MENU_ACTION_ID;
    }
    static isWorkPackagesWorkbenchActionRequest(resourcePathElems) {
        if (!DialogProxyTools.isPostWorkbenchAction(resourcePathElems)) {
            return false;
        }
        const pathFields = DialogProxyTools.deconstructPostWorkbenchActionPath(resourcePathElems);
        return pathFields.actionId === SdaDialogDelegateTools.WORK_PACKAGES_WORKBENCH_ACTION_ID;
    }
    static isWorkPackagesRootDialog(jsonObject) {
        if (!jsonObject || !jsonObject.type || !jsonObject.dialogName) {
            return false;
        }
        return (jsonObject.type === this.EDITOR_DIALOG_MODEL_TYPE &&
            jsonObject.dialogName === this.WORK_PACKAGES_ROOT_DIALOG_NAME);
    }
    static insertBriefcaseMetaDataIntoWorkPackagesDialog(originalDialog) {
        const workPackagesQueryDialog = originalDialog.children[0];
        const propertyDefs = workPackagesQueryDialog.recordDef.propertyDefs;
        propertyDefs.push({
            writeAllowed: false,
            propertyName: 'briefcase',
            canCauseSideEffects: false,
            upperCaseOnly: false,
            propertyType: 'boolean',
            type: 'hxgn.api.dialog.PropertyDef',
            writeEnabled: false
        });
        const columns = workPackagesQueryDialog.view.columns;
        columns.push({
            propertyName: 'briefcase',
            heading: 'Briefcase',
            type: 'hxgn.api.dialog.Column'
        });
        // Return original dialog WITH patches
        return originalDialog;
    }
    static mapDialogNameToOfflineDialogId(name) {
        const offlineDialogId = this.DIALOG_NAME_TO_OFFLINE_DIALOG_ID_MAP[name];
        return offlineDialogId ? offlineDialogId : name;
    }
    static readDialogDelegateStateVisitor(tenantId, userId) {
        const key = this.createStorageKey(tenantId, userId, this.DIALOG_DELEGATE_STATE_KEY);
        return storage.getJson(key).then(jsonObject => {
            if (!jsonObject) {
                const briefcase = SdaGetBriefcaseRecordJsonSample.copyOfResponse();
                BriefcaseVisitor.visitAndSetOnline(briefcase, true);
                jsonObject = {
                    briefcase: SdaGetBriefcaseRecordJsonSample.copyOfResponse(),
                    selectedWorkPackageIds: [],
                    userId: null,
                    workPackages: RecordSetVisitor.emptyRecordSetVisitor().enclosedJsonObject()
                };
                Log.info('SdaDialogDelegateTools::readDelegateState -- returning defaults: ' + JSON.stringify(jsonObject));
            }
            return new SdaDialogDelegateStateVisitor(jsonObject);
        });
    }
    static readOfflineDocumentContentChunk(tenantId, userId, contentId, sequence) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.documents.content.${contentId}.${sequence}`;
        return storage.getJson(key).then(jsonObject => new LargePropertyVisitor(jsonObject));
    }
    static readOfflineDocumentContentRedirection(tenantId, userId, documentsListDialogId, documentId) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.documents.${documentsListDialogId}.alias_OpenLatestFile(${documentId}).redirection`;
        return storage.getJson(key).then(jsonObject => new ContentRedirectionVisitor(jsonObject));
    }
    static readOfflineDocumentsListRecordSet(tenantId, userId, dialogId) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.documents.dialog.${dialogId}.recordSet`;
        return storage.getJson(key).then(jsonObject => new RecordSetVisitor(jsonObject));
    }
    static readOfflineDocumentsPropertiesRecord(tenantId, userId, dialogId) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.documents.dialog.${dialogId}.record`;
        return storage.getJson(key).then(jsonObject => new RecordVisitor(jsonObject));
    }
    static readOfflineDocumentsRedirection(tenantId, userId, workPackageId) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_DOCUMENTS_REDIRECTION_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.getJson(key).then(jsonObject => new DialogRedirectionVisitor(jsonObject));
    }
    static readOfflineDocumentsRootDialog(tenantId, userId, dialogId) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.documents.dialog.${dialogId}`;
        return storage.getJson(key).then(jsonObject => new DialogVisitor(jsonObject));
    }
    static readOfflineSession(tenantId, userId) {
        const key = this.createStorageKey(tenantId, userId, this.OFFLINE_SESSION_KEY);
        return storage.getJson(key).then(jsonObject => new SessionVisitor(jsonObject));
    }
    static readOfflineTagsListRecordSet(tenantId, userId, dialogId) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.tags.dialog.${dialogId}.recordSet`;
        return storage.getJson(key).then(jsonObject => new RecordSetVisitor(jsonObject));
    }
    static readOfflineTagsPropertiesRecord(tenantId, userId, dialogId) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.tags.dialog.${dialogId}.record`;
        return storage.getJson(key).then(jsonObject => new RecordVisitor(jsonObject));
    }
    static readOfflineTagsRedirection(tenantId, userId, workPackageId) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_TAGS_REDIRECTION_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.getJson(key).then(jsonObject => new DialogRedirectionVisitor(jsonObject));
    }
    static readOfflineTagsRootDialog(tenantId, userId, dialogId) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.tags.dialog.${dialogId}`;
        return storage.getJson(key).then(jsonObject => new DialogVisitor(jsonObject));
    }
    static readOfflineWorkPackagesRedirection(tenantId, userId) {
        const key = this.createStorageKey(tenantId, userId, this.OFFLINE_WORK_PACKAGES_REDIRECTION_KEY);
        return storage.getJson(key).then(jsonObject => new DialogRedirectionVisitor(jsonObject));
    }
    static readOfflineWorkPackagesRootDialog(tenantId, userId) {
        const key = this.createStorageKey(tenantId, userId, this.OFFLINE_WORK_PACKAGES_DIALOG_ROOT_KEY);
        return storage.getJson(key).then(jsonObject => new DialogVisitor(jsonObject));
    }
    static showAllStorageKeys() {
        return storage
            .getAllKeys()
            .then(allKeys => {
            const keyCount = allKeys.length;
            Log.info(`SdaDialogDelegateTools::showAllStorageKeys -- key count: ${allKeys.length}`);
            for (let i = keyCount - 1; i > -1; --i) {
                Log.info(`SdaDialogDelegateTools::showAllStorageKeys -- key[${i}]: ${allKeys[i]}`);
            }
        })
            .catch(allKeysError => {
            Log.error('SdaDialogDelegateTools::showAllStorageKeys -- error getting all keys from storage: ' + allKeysError);
        });
    }
    static writeDialogDelegateState(tenantId, stateVisitor) {
        const userId = stateVisitor.visitUserId();
        const key = this.createStorageKey(tenantId, userId, this.DIALOG_DELEGATE_STATE_KEY);
        return storage.setJson(key, stateVisitor.enclosedJsonObject());
    }
    static writeOfflineDocumentContentChunk(tenantId, userId, offlineDocumentsListDialogId, documentId, sequence, largePropertyVisitor) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.documents.content.${offlineDocumentsListDialogId}_alias_OpenLatestFile_${documentId}.${sequence}`;
        return storage.setJson(key, largePropertyVisitor.enclosedJsonObject());
    }
    static writeOfflineDocumentContentRedirection(tenantId, userId, documentsListDialogId, documentId, contentRedirectionVisitor) {
        const key = `${tenantId}.${userId}.ppm.sda.workPackages.documents.${documentsListDialogId}.alias_OpenLatestFile(${documentId}).redirection`;
        return storage.setJson(key, contentRedirectionVisitor.enclosedJsonObject());
    }
    static writeOfflineDocumentsDialogListRecordSet(tenantId, userId, workPackageId, recordSetVisitor) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_DOCUMENTS_DIALOG_LIST_RECORD_SET_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.setJson(key, recordSetVisitor.enclosedJsonObject());
    }
    static writeOfflineDocumentsDialogPropertiesRecord(tenantId, userId, workPackageId, recordVisitor) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_RECORD_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.setJson(key, recordVisitor.enclosedJsonObject());
    }
    static writeOfflineDocumentsDialogRoot(tenantId, userId, workPackageId, dialogVisitor) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_DOCUMENTS_DIALOG_ROOT_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.setJson(key, dialogVisitor.enclosedJsonObject());
    }
    static writeOfflineDocumentsRedirection(tenantId, userId, workPackageId, dialogRedirectionVistor) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_DOCUMENTS_REDIRECTION_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.setJson(key, dialogRedirectionVistor.enclosedJsonObject());
    }
    static writeOfflineSession(tenantId, userId, offlineSessionVisitor) {
        const key = this.createStorageKey(tenantId, userId, this.OFFLINE_SESSION_KEY);
        return storage.setJson(key, offlineSessionVisitor.enclosedJsonObject());
    }
    static writeOfflineTagsDialogListRecordSet(tenantId, userId, workPackageId, recordSetVisitor) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_TAGS_DIALOG_LIST_RECORD_SET_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.setJson(key, recordSetVisitor.enclosedJsonObject());
    }
    static writeOfflineTagsDialogPropertiesRecord(tenantId, userId, workPackageId, recordVisitor) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_TAGS_DIALOG_PROPERTIES_RECORD_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.setJson(key, recordVisitor.enclosedJsonObject());
    }
    static writeOfflineTagsDialogRoot(tenantId, userId, workPackageId, dialogVisitor) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_TAGS_DIALOG_ROOT_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.setJson(key, dialogVisitor.enclosedJsonObject());
    }
    static writeOfflineTagsRedirection(tenantId, userId, workPackageId, dialogRedirectionVistor) {
        let key = this.createStorageKey(tenantId, userId, this.OFFLINE_TAGS_REDIRECTION_KEY);
        key = key.replace('${workPackageId}', workPackageId);
        return storage.setJson(key, dialogRedirectionVistor.enclosedJsonObject());
    }
    static writeOfflineWorkPackagesDialogRoot(tenantId, userId, dialogVisitor) {
        const key = this.createStorageKey(tenantId, userId, this.OFFLINE_WORK_PACKAGES_DIALOG_ROOT_KEY);
        return storage.setJson(key, dialogVisitor.enclosedJsonObject());
    }
    static writeOfflineWorkPackagesRedirection(tenantId, userId, dialogRedirectionVistor) {
        const key = this.createStorageKey(tenantId, userId, this.OFFLINE_WORK_PACKAGES_REDIRECTION_KEY);
        return storage.setJson(key, dialogRedirectionVistor.enclosedJsonObject());
    }
    static constructOfflineModeNullRedirection(tenantId, sessionId, actionId) {
        const nullRedirectionId = DialogProxyTools.constructNullRedirectionId();
        return {
            tenantId: tenantId,
            referringObject: {
                dialogMode: 'READ',
                dialogAlias: 'Briefcase_Briefcase_Details',
                actionId: actionId,
                type: 'hxgn.api.dialog.ReferringDialog',
                dialogId: this.OFFLINE_BRIEFCASE_DIALOG_DETAILS_ID
            },
            refreshNeeded: true,
            sessionId: sessionId,
            id: nullRedirectionId,
            type: 'hxgn.api.dialog.NullRedirection'
        };
    }
    static createStorageKey(tenantId, userId, keyTemplate) {
        const key = keyTemplate.replace('${tenantId}', tenantId);
        return key.replace('${userId}', userId);
    }
}
// Dialog Ids
SdaDialogDelegateTools.OFFLINE_BRIEFCASE_DIALOG_COMMENTS_ID = 'offline_briefcase_comments';
SdaDialogDelegateTools.OFFLINE_BRIEFCASE_DIALOG_DETAILS_ID = 'offline_briefcase_details';
SdaDialogDelegateTools.OFFLINE_BRIEFCASE_DIALOG_ROOT_ID = 'offline_briefcase';
SdaDialogDelegateTools.OFFLINE_BRIEFCASE_DIALOG_WORK_PACKAGES_ID = 'offline_briefcase_workPackages';
SdaDialogDelegateTools.OFFLINE_WORK_PACKAGES_DIALOG_LIST_ID = 'offline_workPackages_list';
SdaDialogDelegateTools.OFFLINE_WORK_PACKAGES_DIALOG_ROOT_ID = 'offline_workPackages';
// Dialog Names
SdaDialogDelegateTools.BRIEFCASE_DETAILS_DIALOG_NAME = 'Briefcase_Briefcase_Details';
SdaDialogDelegateTools.BRIEFCASE_MOBILE_COMMENTS_DIALOG_NAME = 'Briefcase_Briefcase_MobileComments';
SdaDialogDelegateTools.BRIEFCASE_ROOT_DIALOG_NAME = 'Briefcase_Briefcase_FORM';
SdaDialogDelegateTools.BRIEFCASE_WORK_PACKAGES_DIALOG_NAME = 'Briefcase_Briefcase_Workpackages';
SdaDialogDelegateTools.DOCUMENTS_ROOT_DIALOG_NAME = 'Workpackage_Documents_FORM';
SdaDialogDelegateTools.DOCUMENTS_PROPERTIES_DIALOG_NAME = 'Workpackage_Documents_Properties';
SdaDialogDelegateTools.DOCUMENTS_LIST_DIALOG_NAME = 'Workpackage_Documents_Documents';
SdaDialogDelegateTools.TAGS_ROOT_DIALOG_NAME = 'Workpackage_Tags_FORM';
SdaDialogDelegateTools.TAGS_PROPERTIES_DIALOG_NAME = 'Workpackage_Tags_Properties';
SdaDialogDelegateTools.TAGS_LIST_DIALOG_NAME = 'Workpackage_Tags_Tags';
SdaDialogDelegateTools.WORK_PACKAGES_LIST_DIALOG_NAME = 'Workpackage_General';
SdaDialogDelegateTools.WORK_PACKAGES_ROOT_DIALOG_NAME = 'Workpackage_General_FORM';
// =========================================
SdaDialogDelegateTools.OFFLINE_WORKBENCH_ACTION_MAP = {
    Briefcase: '',
    WorkPackages: ''
};
SdaDialogDelegateTools.DIALOG_NAME_TO_OFFLINE_DIALOG_ID_MAP = {
    Briefcase_Briefcase_FORM: 'ppm_sda_briefcase',
    Briefcase_Briefcase_Details: 'ppm_sda_briefcase_details',
    Briefcase_Briefcase_Workpackages: 'ppm_sda_briefcase_workPackages',
    Briefcase_Briefcase_MobileComments: 'ppm_sda_briefcase_comments',
    Workpackage_Documents_FORM: 'ppm_sda_workPackage_documents',
    Workpackage_Documents_Properties: 'ppm_sda_workPackage_documents_properties',
    Workpackage_Documents_Documents: 'ppm_sda_workPackage_documents_list',
    Workpackage_General_FORM: 'ppm_sda_workPackages',
    Workpackage_General: 'ppm_workPackages_list'
};
SdaDialogDelegateTools.NAVIGATION_DIALOG_KEY = '${tenantId}.${userId}.${state}.${event}.dialog';
SdaDialogDelegateTools.NAVIGATION_REDIRECTION_KEY = '${tenantId}.${userId}.${state}.${event}.redirection';
// =========================================
// Dynamic Dialog Ids -- PRIVATE
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_LIST_ID_PREFIX = 'offline_documents_list_';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_LIST_ID = `${SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_LIST_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID_PREFIX = 'offline_documents_properties_';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID = `${SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID_PREFIX = 'offline_documents_';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID = `${SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_ROOT_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_LIST_ID_PREFIX = 'offline_tags_list_';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_LIST_ID = `${SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_LIST_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_PROPERTIES_ID_PREFIX = 'offline_tags_properties_';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_PROPERTIES_ID = `${SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_PROPERTIES_ID_PREFIX}\${workPackageId}`;
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_ROOT_ID_PREFIX = 'offline_tags_';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_ROOT_ID = `${SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_ROOT_ID_PREFIX}\${workPackageId}`;
// Action Ids
SdaDialogDelegateTools.ADD_TO_BRIEFCASE_MENU_ACTION_ID = 'alias_AddToBriefcase';
SdaDialogDelegateTools.BRIEFCASE_WORKBENCH_ACTION_ID = 'Briefcase';
SdaDialogDelegateTools.ENTER_OFFLINE_MODE_MENU_ACTION_ID = 'alias_EnterOfflineMode';
SdaDialogDelegateTools.EXIT_OFFLINE_MODE_MENU_ACTION_ID = 'alias_ExitOfflineMode';
SdaDialogDelegateTools.OPEN_MENU_ACTION_ID = 'alias_Open';
SdaDialogDelegateTools.OPEN_LATEST_FILE_MENU_ACTION_ID = 'alias_OpenLatestFile';
SdaDialogDelegateTools.REMOVE_FROM_BRIEFCASE_MENU_ACTION_ID = 'alias_RemoveFromBriefcase';
SdaDialogDelegateTools.SHOW_TAGS_MENU_ACTION_ID = 'alias_ShowTags';
SdaDialogDelegateTools.WORK_PACKAGES_WORKBENCH_ACTION_ID = 'WorkPackages';
// Model Types
SdaDialogDelegateTools.EDITOR_DIALOG_MODEL_TYPE = 'hxgn.api.dialog.EditorDialog';
SdaDialogDelegateTools.QUERY_DIALOG_MODEL_TYPE = 'hxgn.api.dialog.QueryDialog';
SdaDialogDelegateTools.RECORD_MODEL_TYPE = 'hxgn.api.dialog.Record';
SdaDialogDelegateTools.RECORD_SET_MODEL_TYPE = 'hxgn.api.dialog.RecordSet';
SdaDialogDelegateTools.SESSION_ID_MODEL_TYPE = 'hxgn.api.dialog.SessionId';
// Property Names
SdaDialogDelegateTools.ONLINE_PROPERTY_NAME = 'online';
// Storage Keys
SdaDialogDelegateTools.DIALOG_DELEGATE_STATE_KEY = '${tenantId}.${userId}.ppm.sda.dialog.delegate.state';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_REDIRECTION_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.documents.redirection.${workPackageId}';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_ROOT_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.documents.dialog.offline_documents_${workPackageId}';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_PROPERTIES_RECORD_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.documents.dialog.offline_documents_properties_${workPackageId}.record';
SdaDialogDelegateTools.OFFLINE_DOCUMENTS_DIALOG_LIST_RECORD_SET_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.documents.dialog.offline_documents_list_${workPackageId}.recordSet';
SdaDialogDelegateTools.OFFLINE_SESSION_KEY = '${tenantId}.${userId}.offline.session';
SdaDialogDelegateTools.OFFLINE_TAGS_REDIRECTION_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.tags.redirection.${workPackageId}';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_ROOT_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.tags.dialog.offline_tags_${workPackageId}';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_PROPERTIES_RECORD_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.tags.dialog.offline_tags_properties_${workPackageId}.record';
SdaDialogDelegateTools.OFFLINE_TAGS_DIALOG_LIST_RECORD_SET_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.tags.dialog.offline_tags_list_${workPackageId}.recordSet';
SdaDialogDelegateTools.OFFLINE_WORK_PACKAGES_LIST_RECORD_SET_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.dialog.offline_workPackages_list.recordSet';
SdaDialogDelegateTools.OFFLINE_WORK_PACKAGES_REDIRECTION_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.redirection';
SdaDialogDelegateTools.OFFLINE_WORK_PACKAGES_DIALOG_ROOT_KEY = '${tenantId}.${userId}.ppm.sda.workPackages.dialog.root';
