import { JsonClientResponse } from '../client/JsonClientResponse';
import { storage } from '../storage';
import { Log } from '../util/Log';
import { FetchClient } from '../ws/FetchClient';
export class DialogProxyTools {
    static commonFetchClient() {
        return this.COMMON_FETCH_CLIENT;
    }
    static constructDialogMessageModel(message) {
        return { type: this.DIALOG_MESSAGE_MODEL_TYPE, message };
    }
    static constructLoginModel(userId, password) {
        return {
            userId: 'jordan',
            password: 'jordan1',
            clientType: 'MOBILE',
            deviceProperties: {},
            type: this.LOGIN_MODEL_TYPE
        };
    }
    static constructRequestNotValidDuringOfflineMode(action, resourcePath) {
        return Promise.resolve(new JsonClientResponse(this.constructDialogMessageModel(`${action} at ${resourcePath} is not valid during offline mode: `), 400));
    }
    static constructNullRedirectionId() {
        return `null_redirection__offline_${Date.now()}`;
    }
    static deconstructDeleteSessionPath(path) {
        return {
            tenantId: path[1],
            sessionId: path[3]
        };
    }
    static deconstructGetDialogPath(path) {
        return {
            tenantId: path[1],
            sessionId: path[3],
            dialogId: path[5]
        };
    }
    static deconstructGetRecordPath(path) {
        return {
            tenantId: path[1],
            sessionId: path[3],
            dialogId: path[5]
        };
    }
    static deconstructGetRedirectionPath(path) {
        return {
            tenantId: path[1],
            sessionId: path[3],
            redirectionId: path[5]
        };
    }
    static deconstructPostMenuActionPath(path) {
        return {
            tenantId: path[1],
            sessionId: path[3],
            dialogId: path[5],
            actionId: path[7]
        };
    }
    static deconstructPostRecordsPath(path) {
        return {
            tenantId: path[1],
            sessionId: path[3],
            dialogId: path[5]
        };
    }
    static deconstructPostSessionContentPath(path) {
        return {
            tenantId: path[1],
            sessionId: path[3],
            contentId: path[5]
        };
    }
    static deconstructPostSessionsPath(path) {
        return {
            tenantId: path[1]
        };
    }
    static deconstructPostWorkbenchActionPath(path) {
        return {
            tenantId: path[1],
            sessionId: path[3],
            workbenchId: path[5],
            actionId: path[7]
        };
    }
    static deleteAllDialogState(tenantId, userId, dialogId) {
        const dialog = this.readDialogState(tenantId, userId, dialogId);
        if (dialog) {
            this.deleteAllDialogStateFor(tenantId, userId, dialog);
        }
        else {
            this.deleteRedirectionState(tenantId, userId, dialogId);
        }
    }
    static deleteAllDialogStateFor(tenantId, userId, dialog) {
        const dialogChildren = dialog.children;
        if (dialogChildren) {
            for (const child of dialogChildren) {
                this.deleteAllDialogStateFor(tenantId, userId, child);
            }
        }
        this.deleteRedirectionState(tenantId, userId, dialog.id);
        this.deleteDialogState(tenantId, userId, dialog.id);
        this.deleteRecordSetState(tenantId, userId, dialog.id);
        this.deleteRecordState(tenantId, userId, dialog.id);
        this.deleteDialogParentState(tenantId, userId, dialog.id);
        this.deleteDialogAliasState(tenantId, userId, dialog.id);
        this.deleteDialogReferringAliasState(tenantId, userId, dialog.id);
    }
    static deleteAllState(tenantId, userId) {
        storage
            .getAllKeys()
            .then(allKeys => {
            const keyCount = allKeys.length;
            for (let i = keyCount - 1; i > -1; --i) {
                const key = allKeys[i];
                Log.debug('DialogProxyTools::deleteAllState -- removing from storage: ' + key);
                storage.removeItem(key).catch(removeItemError => {
                    Log.error('DialogProxyTools::deleteAllState -- error removing item: ' + removeItemError);
                });
            }
        })
            .catch(allKeysError => {
            Log.error('DialogProxyTools::deleteAllState -- error getting all keys from storage: ' + allKeysError);
        });
    }
    static deleteAllWorkbenchNavigation(tenantId, userId, navigationKey) {
        const previousNavigation = DialogProxyTools.readNavigationState(tenantId, userId, navigationKey);
        if (previousNavigation) {
            DialogProxyTools.deleteAllDialogState(tenantId, userId, previousNavigation.redirectionId);
        }
    }
    static deleteDialogAliasState(tenantId, userId, dialogId) {
        this.deletePersistentState(tenantId, userId, 'dialog.' + dialogId + '.alias');
    }
    static deleteDialogParentState(tenantId, userId, childId) {
        this.deletePersistentState(tenantId, userId, 'dialog.' + childId + '.parent');
    }
    static deleteDialogReferringAliasState(tenantId, userId, dialogId) {
        this.deletePersistentState(tenantId, userId, 'dialog.' + dialogId + '.referringAlias');
    }
    static deleteDialogState(tenantId, userId, dialogId) {
        this.deletePersistentState(tenantId, userId, 'dialog.' + dialogId);
    }
    static deleteNavigationState(tenantId, userId, navigationId) {
        this.deletePersistentState(tenantId, userId, 'navigation.' + navigationId);
    }
    static deletePersistentState(tenantId, userId, stateId) {
        const key = tenantId + '.' + userId + '.' + stateId;
        storage
            .removeItem(key)
            .catch(removeItemError => Log.error('Error removing item from storage: ' + removeItemError));
    }
    static deleteRecordSetState(tenantId, userId, dialogId) {
        this.deletePersistentState(tenantId, userId, 'dialog.' + dialogId + '.recordSet');
    }
    static deleteRecordState(tenantId, userId, dialogId) {
        this.deletePersistentState(tenantId, userId, 'dialog.' + dialogId + '.record');
    }
    static deleteRedirectionState(tenantId, userId, redirectionId) {
        this.deletePersistentState(tenantId, userId, 'redirection.' + redirectionId);
    }
    static deleteSessionState(tenantId, userId) {
        this.deletePersistentState(tenantId, userId, 'session');
    }
    static findDialogState(tenantId, userId, dialogId) {
        const dialog = this.findRootDialogState(tenantId, userId, dialogId);
        if (!dialog) {
            return null;
        }
        return this.findDialogStateWithin(tenantId, userId, dialog, dialogId);
    }
    static findDialogStateWithin(tenantId, userId, dialog, targetId) {
        if (dialog && dialog.id === targetId) {
            return dialog;
        }
        const dialogChildren = dialog.children;
        if (dialogChildren) {
            for (const child of dialogChildren) {
                const nestedDialog = this.findDialogStateWithin(tenantId, userId, child, targetId);
                if (nestedDialog) {
                    return nestedDialog;
                }
            }
        }
        return null;
    }
    static findRecordProperty(record, propertyName) {
        for (const p of record.properties) {
            if (p.name === propertyName) {
                return p;
            }
        }
        return null;
    }
    static findRecordPropertyValue(record, propertyName) {
        const p = this.findRecordProperty(record, propertyName);
        return p ? p.value : null;
    }
    static findRootDialogState(tenantId, userId, dialogId) {
        const dialog = this.readDialogState(tenantId, userId, dialogId);
        if (dialog) {
            return dialog;
        }
        const parentId = this.readDialogParentState(tenantId, userId, dialogId);
        if (parentId) {
            return this.findRootDialogState(tenantId, userId, parentId);
        }
        return null;
    }
    static isCreateSessionRequest(path) {
        return path.length === 3 && path[0] === DialogProxyTools.TENANTS && path[2] === DialogProxyTools.SESSIONS;
    }
    static isDeleteSessionRequest(path) {
        return path.length === 4 && path[0] === DialogProxyTools.TENANTS && path[2] === DialogProxyTools.SESSIONS;
    }
    static isGetAvailableViews(path) {
        return (path.length === 7 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.AVAILABLE_VIEWS);
    }
    static isGetDialog(path) {
        return (path.length === 6 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS);
    }
    static isGetRecord(path) {
        return (path.length === 7 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.RECORD);
    }
    static isGetRedirection(path) {
        return (path.length === 6 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.REDIRECTIONS);
    }
    static isGetSelectedView(path) {
        return (path.length === 7 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.SELECTED_VIEW);
    }
    static isGetSession(path) {
        return path.length === 4 && path[0] === DialogProxyTools.TENANTS && path[2] === DialogProxyTools.SESSIONS;
    }
    static isGetTenant(path) {
        return path.length === 2 && path[0] === DialogProxyTools.TENANTS;
    }
    static isGetTenants(path) {
        return path.length === 1 && path[0] === DialogProxyTools.TENANTS;
    }
    static isGetViewMode(path) {
        return (path.length === 7 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.VIEW_MODE);
    }
    static isGetWorkbenches(path) {
        return (path.length === 5 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.WORKBENCHES);
    }
    static isPostAvailableValues(path) {
        return (path.length === 9 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.RECORD &&
            path[8] === DialogProxyTools.AVAILABLE_VALUES);
    }
    static isPostMenuAction(path) {
        return (path.length === 8 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.ACTIONS);
    }
    static isPostRecords(path) {
        return (path.length === 7 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.RECORDS);
    }
    static isPostSession(path) {
        return path.length === 3 && path[0] === DialogProxyTools.TENANTS && path[2] === DialogProxyTools.SESSIONS;
    }
    static isPostSessionContent(path) {
        return (path.length === 6 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.CONTENT);
    }
    static isPostWorkbenchAction(path) {
        return (path.length === 8 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.WORKBENCHES &&
            path[6] === DialogProxyTools.ACTIONS);
    }
    static isPutRecord(path) {
        return (path.length === 7 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.RECORD);
    }
    static isPutSelectedView(path) {
        return (path.length === 8 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.SELECTED_VIEW);
    }
    static isPutViewMode(path) {
        return (path.length === 8 &&
            path[0] === DialogProxyTools.TENANTS &&
            path[2] === DialogProxyTools.SESSIONS &&
            path[4] === DialogProxyTools.DIALOGS &&
            path[6] === DialogProxyTools.VIEW_MODE);
    }
    static isActionParametersObject(jsonObject) {
        if (!jsonObject || !jsonObject['type']) {
            return false;
        }
        return jsonObject['type'] === this.ACTION_PARAMETERS_MODEL_TYPE;
    }
    static isAnnotationObject(jsonObject) {
        if (!jsonObject || !jsonObject['type']) {
            return false;
        }
        return jsonObject['type'] === this.ANNOTATION_MODEL_TYPE;
    }
    static isDialogObject(jsonObject) {
        if (!jsonObject || !jsonObject['type']) {
            return false;
        }
        return (jsonObject['type'] === this.EDITOR_DIALOG_MODEL_TYPE || jsonObject['type'] === this.QUERY_DIALOG_MODEL_TYPE);
    }
    static isLoginObject(jsonObject) {
        if (!jsonObject || !jsonObject['type']) {
            return false;
        }
        return jsonObject['type'] === this.LOGIN_MODEL_TYPE;
    }
    static isPropertyObject(jsonObject) {
        if (!jsonObject || !jsonObject['type']) {
            return false;
        }
        return jsonObject['type'] === this.PROPERTY_MODEL_TYPE;
    }
    static isPropertyDefObject(jsonObject) {
        if (!jsonObject || !jsonObject['type']) {
            return false;
        }
        return jsonObject['type'] === this.PROPERTY_DEF_MODEL_TYPE;
    }
    static isRecordObject(jsonObject) {
        if (!jsonObject || !jsonObject['type']) {
            return false;
        }
        return jsonObject['type'] === this.RECORD_MODEL_TYPE;
    }
    static isRecordSetObject(jsonObject) {
        if (!jsonObject || !jsonObject['type']) {
            return false;
        }
        return jsonObject['type'] === this.RECORD_SET_MODEL_TYPE;
    }
    static isSessionObject(jsonObject) {
        if (!jsonObject || !jsonObject['type']) {
            return false;
        }
        return jsonObject['type'] === this.SESSION_MODEL_TYPE;
    }
    static readDialogAliasState(tenantId, userId, dialogId) {
        return this.readPersistentState(tenantId, userId, 'dialog.' + dialogId + '.alias');
    }
    static readDialogParentState(tenantId, userId, childId) {
        return this.readPersistentState(tenantId, userId, 'dialog.' + childId + '.parent');
    }
    static readDialogReferringAliasState(tenantId, userId, dialogId) {
        return this.readPersistentState(tenantId, userId, 'dialog.' + dialogId + '.referringAlias');
    }
    static readDialogState(tenantId, userId, dialogId) {
        return this.readPersistentState(tenantId, userId, 'dialog.' + dialogId);
    }
    static readNavigationState(tenantId, userId, navigationId) {
        return this.readPersistentState(tenantId, userId, 'navigation.' + navigationId);
    }
    static readPersistentState(tenantId, userId, stateId) {
        const key = tenantId + '.' + userId + '.' + stateId;
        return storage
            .getItem(key)
            .then(json => {
            return json ? JSON.parse(json) : null;
        })
            .catch(getItemError => 'Error getting item from storage: ' + getItemError);
    }
    static readRecordSetState(tenantId, userId, dialogId) {
        return this.readPersistentState(tenantId, userId, 'dialog.' + dialogId + '.recordSet');
    }
    static readRecordState(tenantId, userId, dialogId) {
        return this.readPersistentState(tenantId, userId, 'dialog.' + dialogId + '.record');
    }
    static readRedirectionState(tenantId, userId, redirectionId) {
        return this.readPersistentState(tenantId, userId, 'redirection.' + redirectionId);
    }
    static readSessionState(tenantId, userId) {
        return this.readPersistentState(tenantId, userId, 'session');
    }
    static writeAllDialogParentState(tenantId, userId, parent) {
        const dialogChildren = parent.children;
        if (dialogChildren) {
            for (let i = 0; i < dialogChildren.length; i++) {
                const child = dialogChildren[i];
                this.writeDialogParentState(tenantId, userId, child, i, parent);
            }
        }
    }
    static writeDialogAliasState(tenantId, userId, dialogId, alias) {
        return this.writePersistentState(tenantId, userId, 'dialog.' + dialogId + '.alias', alias);
    }
    static writeDialogParentState(tenantId, userId, child, childIndex, parent) {
        this.writePersistentState(tenantId, userId, 'dialog.' + child.id + '.parent', parent.id);
        const parentAlias = this.readDialogAliasState(tenantId, userId, parent.id);
        this.writeDialogAliasState(tenantId, userId, child.id, parentAlias + '_' + childIndex);
        this.writeAllDialogParentState(tenantId, userId, child);
    }
    static writeDialogReferringAliasState(tenantId, userId, dialogId, referringAlias) {
        this.writePersistentState(tenantId, userId, 'dialog.' + dialogId + '.referringAlias', referringAlias);
    }
    static writeDialogState(tenantId, userId, dialog) {
        return this.writePersistentState(tenantId, userId, 'dialog.' + dialog.id, dialog);
    }
    static writePersistentState(tenantId, userId, stateId, state) {
        const key = tenantId + '.' + userId + '.' + stateId;
        return storage
            .setItem(key, JSON.stringify(state))
            .then(value => {
            return true;
        })
            .catch(setItemError => {
            Log.error('Error setting item in storage: ' + setItemError);
            return false;
        });
    }
    static writeRecordSetState(tenantId, userId, dialogId, recordSet) {
        return this.writePersistentState(tenantId, userId, 'dialog.' + dialogId + '.recordSet', recordSet);
    }
    static writeRecordState(tenantId, userId, dialogId, record) {
        return this.writePersistentState(tenantId, userId, 'dialog.' + dialogId + '.record', record);
    }
    static writeNavigationState(tenantId, userId, navigation) {
        return this.writePersistentState(tenantId, userId, 'navigation.' + navigation.id, navigation);
    }
    static writeRedirectionState(tenantId, userId, redirection) {
        return this.writePersistentState(tenantId, userId, 'redirection.' + redirection.id, redirection);
    }
    static writeSessionState(session) {
        return this.writePersistentState(session.tenantId, session.userId, 'session', session);
    }
}
DialogProxyTools.ACTIONS = 'actions';
DialogProxyTools.AVAILABLE_VALUES = 'availableValues';
DialogProxyTools.AVAILABLE_VIEWS = 'availableViews';
DialogProxyTools.CONTENT = 'content';
DialogProxyTools.DIALOGS = 'dialogs';
DialogProxyTools.RECORD = 'record';
DialogProxyTools.RECORDS = 'records';
DialogProxyTools.REDIRECTIONS = 'redirections';
DialogProxyTools.SELECTED_VIEW = 'selectedView';
DialogProxyTools.SESSIONS = 'sessions';
DialogProxyTools.TENANTS = 'tenants';
DialogProxyTools.VIEW_MODE = 'viewMode';
DialogProxyTools.WORKBENCHES = 'workbenches';
// Model Types
DialogProxyTools.ACTION_PARAMETERS_MODEL_TYPE = 'hxgn.api.dialog.ActionParameters';
DialogProxyTools.ANNOTATION_MODEL_TYPE = 'hxgn.api.dialog.Annotation';
DialogProxyTools.DIALOG_MESSAGE_MODEL_TYPE = 'hxgn.api.dialog.DialogMessage';
DialogProxyTools.EDITOR_DIALOG_MODEL_TYPE = 'hxgn.api.dialog.EditorDialog';
DialogProxyTools.LOGIN_MODEL_TYPE = 'hxgn.api.dialog.Login';
DialogProxyTools.PROPERTY_MODEL_TYPE = 'hxgn.api.dialog.Property';
DialogProxyTools.PROPERTY_DEF_MODEL_TYPE = 'hxgn.api.dialog.PropertyDef';
DialogProxyTools.QUERY_DIALOG_MODEL_TYPE = 'hxgn.api.dialog.QueryDialog';
DialogProxyTools.RECORD_MODEL_TYPE = 'hxgn.api.dialog.Record';
DialogProxyTools.RECORD_SET_MODEL_TYPE = 'hxgn.api.dialog.RecordSet';
DialogProxyTools.SESSION_MODEL_TYPE = 'hxgn.api.dialog.Session';
DialogProxyTools.COMMON_FETCH_CLIENT = new FetchClient();
