import { DialogProxyTools } from './DialogProxyTools';
/**
 *
 */
export class DialogVisitor {
    constructor(value) {
        if (typeof value === 'string') {
            this._enclosedJsonObject = JSON.parse(value);
        }
        else {
            this._enclosedJsonObject = value;
        }
        if (!DialogProxyTools.isDialogObject(this._enclosedJsonObject)) {
            throw new Error('Object passed to DialogVisitor is not a Dialog');
        }
    }
    // --- State Management Helpers --- //
    static propagateTenantIdAndSessionId(dialog, tenantId, sessionId) {
        new DialogVisitor(dialog).propagateTenantIdAndSessionId(tenantId, sessionId);
    }
    static visitId(dialog) {
        return new DialogVisitor(dialog).visitId();
    }
    // --- State Import/Export --- //
    copyAsJsonObject() {
        return JSON.parse(this.copyAsJsonString());
    }
    copyAsJsonString() {
        return JSON.stringify(this.enclosedJsonObject());
    }
    enclosedJsonObject() {
        return this._enclosedJsonObject;
    }
    // --- State Management --- //
    propagateTenantIdAndSessionId(tenantId, sessionId) {
        this.enclosedJsonObject()['tenantId'] = tenantId;
        this.enclosedJsonObject()['sessionId'] = sessionId;
        if (this.enclosedJsonObject()['children']) {
            for (const c of this.enclosedJsonObject()['children']) {
                const dialogVisitor = new DialogVisitor(c);
                dialogVisitor.propagateTenantIdAndSessionId(tenantId, sessionId);
            }
        }
    }
    visitDialogName() {
        return this.enclosedJsonObject().dialogName;
    }
    visitId() {
        return this.enclosedJsonObject().id;
    }
    visitAndSetId(id) {
        this.enclosedJsonObject().id = id;
    }
    visitChildAt(index) {
        return new DialogVisitor(this.enclosedJsonObject().children[index]);
    }
    visitChildAtName(name) {
        if (this.enclosedJsonObject()['children']) {
            for (const c of this.enclosedJsonObject()['children']) {
                const dialogVisitor = new DialogVisitor(c);
                if (dialogVisitor.visitDialogName() && dialogVisitor.visitDialogName() === name) {
                    return dialogVisitor;
                }
                const childDialogVisitor = dialogVisitor.visitChildAtName(name);
                if (childDialogVisitor) {
                    return childDialogVisitor;
                }
            }
        }
        return null;
    }
    visitChildAtNameAndSetId(name, id) {
        const childDialogVisitor = this.visitChildAtName(name);
        if (childDialogVisitor) {
            childDialogVisitor.visitAndSetId(id);
            return true;
        }
        return false;
    }
}
