import { BlobClientResponse } from '../client/BlobClientResponse';
import { JsonClientResponse } from '../client/JsonClientResponse';
import { TextClientResponse } from '../client/TextClientResponse';
import { VoidClientResponse } from '../client/VoidClientResponse';
import { StreamProducer } from '../io/StreamProducer';
import { DialogDelegate } from '../proxy/DialogDelegate';
import { StringDictionary } from '../util/StringDictionary';
export declare class SdaDialogDelegate implements DialogDelegate {
    private _dialogDelegateStateVisitor;
    private _lastActivity;
    readonly lastActivity: Date;
    initialize(): Promise<void>;
    getBlob(baseUrl: string, resourcePath?: string): Promise<BlobClientResponse> | null;
    getText(baseUrl: string, resourcePath?: string): Promise<TextClientResponse> | null;
    openStream(baseUrl: string, resourcePath?: string): Promise<StreamProducer> | null;
    postMultipart<T>(baseUrl: string, resourcePath: string, formData: FormData): Promise<VoidClientResponse> | null;
    getJson(baseUrl: string, resourcePath: string, queryParams?: StringDictionary): Promise<JsonClientResponse> | null;
    postJson(baseUrl: string, resourcePath: string, body?: StringDictionary): Promise<JsonClientResponse> | null;
    putJson(baseUrl: string, resourcePath: string, body?: StringDictionary): Promise<JsonClientResponse> | null;
    deleteJson(baseUrl: string, resourcePath: string): Promise<JsonClientResponse> | null;
    handleDeleteJsonResponse(baseUrl: string, resourcePath: string, response: Promise<JsonClientResponse>): Promise<JsonClientResponse> | null;
    handleGetBlobResponse(baseUrl: string, resourcePath: string, response: Promise<BlobClientResponse>): Promise<BlobClientResponse> | null;
    handleGetJsonResponse(baseUrl: string, resourcePath: string, queryParams: StringDictionary, response: Promise<JsonClientResponse>): Promise<JsonClientResponse> | null;
    handleGetTextResponse(baseUrl: string, resourcePath: string, response: Promise<TextClientResponse>): Promise<TextClientResponse> | null;
    handleOpenStreamResponse(baseUrl: string, resourcePath: string, response: Promise<StreamProducer>): Promise<StreamProducer> | null;
    handlePostJsonResponse(baseUrl: string, resourcePath: string, body: StringDictionary, response: Promise<JsonClientResponse>): Promise<JsonClientResponse> | null;
    handlePostMultipartResponse<T>(baseUrl: string, resourcePath: string, formData: FormData, response: Promise<VoidClientResponse>): Promise<VoidClientResponse> | null;
    handlePutJsonResponse(baseUrl: string, resourcePath: string, body: StringDictionary, response: Promise<JsonClientResponse>): Promise<JsonClientResponse> | null;
    private captureOfflineSession(baseUrl, tenantId, sessionId);
    private captureNextOfflineWorkPackage(onlineWorkPackagesListDialogId, nextWorkPackageId, tenantId, sessionId);
    private captureNextOfflineTags(onlineDocumentsListDialogId, offlineDocumentsListDialogId, nextWorkPackageId, tenantId, sessionId);
    private captureNextOfflineDocumentContent(onlineDocumentsListDialogId, offlineDocumentsListDialogId, nextDocumentRecordVisitor, tenantId, sessionId);
    private performLoginForOnlineProcessing(tenantId);
    private captureOfflineWorkPackages(tenantId, sessionId);
    private initializeAfterCreateSession(baseUrl, resourcePathElems, body, sessionVisitor);
    private online();
    private performBriefcaseWorkbenchActionRequest(baseUrl, resourcePathElems, body?);
    private performCreateSessionRequest(baseUrl, resourcePathElems, body?);
    private performDeleteSessionRequest(baseUrl, resourcePathElems, body?);
    private performEnterOfflineModeMenuActionRequest(baseUrl, resourcePathElems, body?);
    private performExitOfflineModeMenuActionRequest(baseUrl, resourcePathElems, body?);
    private performOfflineBriefcaseCommentsRequest(baseUrl, resourcePathElems, body?);
    private performOfflineBriefcaseWorkPackagesRequest(baseUrl, resourcePathElems, body?);
    private performOfflineDocumentContentRequest(baseUrl, resourcePathElems, body);
    private performOfflineDocumentOpenLatestFileMenuActionRequest(baseUrl, resourcePathElems, body);
    private performOfflineDocumentsListRecordSetRequest(baseUrl, resourcePathElems, body);
    private performOfflineDocumentsPropertiesRecordRequest(baseUrl, resourcePathElems);
    private performOfflineDocumentsRootDialogRequest(baseUrl, resourcePathElems);
    private performOfflineShowTagsMenuActionRequest(baseUrl, resourcePathElems, body?);
    private performOfflineTagsListRecordSetRequest(baseUrl, resourcePathElems, body);
    private performOfflineTagsPropertiesRecordRequest(baseUrl, resourcePathElems);
    private performOfflineTagsRootDialogRequest(baseUrl, resourcePathElems);
    private performOfflineWorkPackagesListRecordSetRequest(baseUrl, resourcePathElems, body?);
    private performOfflineWorkPackagesOpenMenuActionRequest(baseUrl, resourcePathElems, body?);
    private performOfflineWorkPackagesRootDialogRequest(baseUrl, resourcePathElems);
    private performOfflineWorkPackagesWorkbenchActionRequest(baseUrl, resourcePathElems, body?);
    private performWorkPackagesAddWorkPackageToBriefcase(baseUrl, resourcePathElems, body?);
    private performWorkPackagesRemoveWorkPackageFromBriefcase(baseUrl, resourcePathElems, body?);
}
